from typer import Typer, Option
from rich import print
from typing_extensions import Annotated


from tizori_cli.wrapper import roles
from tizori_cli.commands.base import base_wrapper

roles_app = Typer()


@roles_app.command("list")
def list_roles():
    try:
        status, roles_list = roles.list_roles(base_wrapper)
        if status:
            print("\n[bold blue]Roles[/bold blue]")
            for role in roles_list:
                print(f"\nName: [green]{role.get('name')}[/green]")
                print(f"ID: [green]{role.get('id')}[/green]")
        else:
            print(f"\n[red]Error[/red]: {roles_list}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@roles_app.command("get")
def get_role(
    role_id: Annotated[str, Option(help="Role ID", prompt=True)],
):
    try:
        status, role = roles.get_role(base_wrapper, role_id)
        if status:
            print("\n[bold blue]Role[/bold blue]")
            print(f"Name: [green]{role.get('name')}[/green]")
            print(f"ID: [green]{role.get('id')}[/green]")
            print("Permissions:")
            for permission in role.get("permissions"):
                print(f"[green]{permission.get('scope')} - {permission.get('permission')}[/green]")
        else:
            print(f"\n[red]Error[/red]: {role}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@roles_app.command("create")
def create_role(
    name: Annotated[str, Option(help="Name", prompt=True)],
):
    try:
        status, message = roles.create_role(base_wrapper, name)
        if status:
            print("\n[green]Role Created![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@roles_app.command("update")
def update_role(
    role_id: Annotated[str, Option(help="Role ID", prompt=True)],
    name: Annotated[str, Option(help="Name", prompt=True)],
):
    try:
        data = {"name": name}
        status, message = roles.update_role(base_wrapper, role_id, data)
        if status:
            print("\n[green]Role Updated![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@roles_app.command("add-permission")
def add_permissions_to_role(
    role_id: Annotated[str, Option(help="Role ID", prompt=True)],
    scope: Annotated[str, Option(help="Scope", prompt=True)],
    permission: Annotated[str, Option(help="Permission", prompt=True)],
):
    try:
        perm = {"scope": scope, "permission": permission}
        status, role = roles.get_role(base_wrapper, role_id)
        if not status:
            print(f"\n[red]Error[/red]: {role}")
            return
        permissions = role.get("permissions")
        if permissions is None:
            permissions = []
        data = {"permissions": permissions + [perm], "name": role.get("name")}
        status, message = roles.update_role(base_wrapper, role_id, data)
        if status:
            print("\n[green]Role Updated![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@roles_app.command("remove-permission")
def remove_permissions_from_role(
    role_id: Annotated[str, Option(help="Role ID", prompt=True)],
    scope: Annotated[str, Option(help="Scope", prompt=True)],
    permission: Annotated[str, Option(help="Permission", prompt=True)],
):
    try:
        perm = {"scope": scope, "permission": permission}
        status, role = roles.get_role(base_wrapper, role_id)
        if not status:
            print(f"\n[red]Error[/red]: {role}")
            return
        data = {"permissions": [p for p in role.get("permissions") if p != perm], "name": role.get("name")}
        status, message = roles.update_role(base_wrapper, role_id, data)
        if status:
            print("\n[green]Role Updated![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@roles_app.command("delete")
def delete_role(
    role_id: Annotated[str, Option(help="Role ID", prompt=True)],
):
    try:
        status, message = roles.delete_role(base_wrapper, role_id)
        if status:
            print("\n[green]Role Deleted![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")



from tizori_cli.config import get_config_value
from tizori_cli.wrapper.base import BaseRequestWrapper

base_wrapper = BaseRequestWrapper(base_url=get_config_value("base_url"), 
                                  bearer_token=get_config_value("token"))


import yaml

def load_config():
    """Load the config file if it exists, otherwise create it"""
    try:
        with open("config.yaml") as f:
            return yaml.safe_load(f)
    except FileNotFoundError:
        return {"base_url": ""}

def save_config(config):
    try:
        with open("config.yaml", "w+") as f:
            yaml.safe_dump(config, f)
    except Exception as e:
        print(f"Error saving config: {e}")

def set_config_value(key, value):
    config = load_config()
    config[key] = value
    save_config(config)

def delete_config_value(key):
    config = load_config()
    config.pop(key, None)
    save_config(config)

def get_config_value(key):
    config = load_config()
    return config.get(key)


def list_users(req_wrap):
    endpoint = "/users"
    response, status = req_wrap.get(endpoint)
    if status == 200:
        users = response
        return True, users
    else:
        return False, response.get("detail", "Unknown Error")

def get_user(req_wrap, username):
    endpoint = f"/users/{username}"
    response, status = req_wrap.get(endpoint)
    if status == 200:
        user = response
        return True, user
    else:
        return False, response.get("detail", "Unknown Error")
    
def create_user(req_wrap, username, email, name):
    endpoint = "/users"
    response, status = req_wrap.post(endpoint, data={"username": username, "email": email, "name": name})
    if status == 201:
        return True, response.get("password")
    else:
        return False, response.get("detail", "Unknown Error")
    
def update_user(req_wrap, username, data):
    endpoint = f"/users/{username}"
    response, status = req_wrap.patch(endpoint, data=data)
    if status == 200:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")

def delete_user(req_wrap, username):
    endpoint = f"/users/{username}"
    response, status = req_wrap.delete(endpoint)
    if status == 200:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")


def list_applications(req_wrap):
    endpoint = "/applications"
    response, status = req_wrap.get(endpoint)
    if status == 200:
        applications = response
        return True, applications
    else:
        return False, response.get("detail", "Unknown Error")
    

def get_application_credentials(req_wrap, application_id):
    endpoint = f"/credentials/{application_id}"
    response, status = req_wrap.get(endpoint)
    if status == 200:
        credentials = response
        return True, credentials
    else:
        return False, response.get("detail", "Unknown Error")


def create_application(req_wrap, name):
    endpoint = "/applications"
    response, status = req_wrap.post(endpoint, data={"name": name})
    if status == 201:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")
    

def update_application_credentials(req_wrap, application_id, data):
    endpoint = f"/credentials/{application_id}"
    response, status = req_wrap.patch(endpoint, data=data)
    if status == 200:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")
    

def delete_application(req_wrap, application_id):
    endpoint = f"/applications/{application_id}"
    response, status = req_wrap.delete(endpoint)
    if status == 200:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")



def list_roles(req_wrap):
    endpoint = "/roles"
    response, status = req_wrap.get(endpoint)
    if status == 200:
        roles = response
        return True, roles
    else:
        return False, response.get("detail", "Unknown Error")

def get_role(req_wrap, role_id):
    endpoint = f"/roles/{role_id}"
    response, status = req_wrap.get(endpoint)
    if status == 200:
        role = response
        return True, role
    else:
        return False, response.get("detail", "Unknown Error")
    
def create_role(req_wrap, name):
    endpoint = "/roles"
    response, status = req_wrap.post(endpoint, data={"name": name})
    if status == 201:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")
    
def update_role(req_wrap, role_id, data):
    endpoint = f"/roles/{role_id}"
    response, status = req_wrap.patch(endpoint, data=data)
    if status == 200:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")

def delete_role(req_wrap, role_id):
    endpoint = f"/roles/{role_id}"
    response, status = req_wrap.delete(endpoint)
    if status == 200:
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")



import string
from typer import Typer, Option
from rich import print
from typing_extensions import Annotated


from tizori_cli.wrapper import users, roles
from tizori_cli.commands.base import base_wrapper


users_app = Typer()

def clean_and_split_roles(roles_str):
    # Define characters to remove
    chars_to_remove = string.whitespace + '[](){}'

    # Create a translation table to map each character in chars_to_remove to None
    translation_table = str.maketrans('', '', chars_to_remove)

    # Remove specified characters from the roles string
    cleaned_roles_str = roles_str.translate(translation_table)

    # Split the cleaned string by commas to get a list of roles
    roles_list = cleaned_roles_str.split(',')

    # Return the list of roles
    return roles_list

@users_app.command("list")
def list_users():
    try:
        status, users_list = users.list_users(base_wrapper)
        if status:
            print("\n[bold blue]Users[/bold blue]")
            for user in users_list:
                print(f"\nUsername: [bold]{user.get('username')}[/bold]")
                print(f"Email: [bold]{user.get('email')}[/bold]")
                print(f"Name: [bold]{user.get('name')}[/bold]")
        else:
            print(f"\n[red]Error[/red]: {users_list}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")

@users_app.command("get")
def get_user(
    username: Annotated[str, Option(help="Username", prompt=True)],
):
    try:
        status, user = users.get_user(base_wrapper, username)
        if status:
            print("\n[bold blue]User[/bold blue]")
            print(f"Username: [green]{user.get('username')}[/green]")
            print(f"Email: [green]{user.get('email')}[/green]")
            print(f"Name: [green]{user.get('name')}[/green]")
            print("Roles:")
            for role in user.get("roles"):
                print(f"[green]{role.get("name")} - {role.get("id")}[/green]")
        else:
            print(f"\n[red]Error[/red]: {user}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")

@users_app.command("create")
def create_user(
    username: Annotated[str, Option(help="Username", prompt=True)],
    email: Annotated[str, Option(help="Email", prompt=True)],
    name: Annotated[str, Option(help="Name", prompt=True)],
):
    try:
        status, message = users.create_user(base_wrapper, username, email, name)
        if status:
            print("\n[green]User Created![/green]")
            # Display generated password
            print(f"Password: [bold blue]{message}[/bold blue]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")

@users_app.command("update")
def update_user(
    username: Annotated[str, Option(help="Username", prompt=True)],
    email: Annotated[str, Option(help="Email", prompt=True)],
    name: Annotated[str, Option(help="Name", prompt=True)],
):
    try:
        data = {"email": email, "name": name}
        status, message = users.update_user(base_wrapper, username, data)
        if status:
            print("\n[green]User Updated![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")

@users_app.command("update-roles")
def update_user_roles(
    username: Annotated[str, Option(help="Username", prompt=True)],
    roles: Annotated[str, Option(help="Roles", prompt=True)],
):
    try:
        roles = clean_and_split_roles(roles)
        # status, roles_list = roles.list_roles(base_wrapper)
        # if not status:
        #     print(f"\n[red]Error[/red]: {roles_list}")
        #     return
        # roles_dict = {role.get("name"): role.get("id") for role in roles_list}
        data = {"roles": roles}
        status, message = users.update_user(base_wrapper, username, data)
        if status:
            print("\n[green]User Roles Updated![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")

@users_app.command("delete")
def delete_user(
    username: Annotated[str, Option(help="Username", prompt=True)],
):
    try:
        status, message = users.delete_user(base_wrapper, username)
        if status:
            print("\n[green]User Deleted![/green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")
    



from typer import Typer, Abort, Context, echo
from rich import print

from tizori_cli.config import load_config, save_config
from tizori_cli.commands.auth import auth_app
from tizori_cli.commands.users import users_app
from tizori_cli.commands.roles import roles_app
from tizori_cli.commands.applications import applications_app


app = Typer(context_settings={"help_option_names": ["-h", "--help"]})
app.add_typer(auth_app, name="auth")
app.add_typer(users_app, name="users")
app.add_typer(roles_app, name="roles")
app.add_typer(applications_app, name="apps")


@app.callback(invoke_without_command=True)
def callback(ctx: Context):
    try:
        config = load_config()

        # Show help if no subcommand is provided
        if ctx.invoked_subcommand is None:
            print(
                """
        [white]███████████████╗[/white]                 
        [white]███╔══════════███╗               
    [white]███╔╝           ███╗      ████████╗██╗███████╗ ██████╗ ██████╗ ██╗[/white]           
    [white]███║            ███║      ╚══██╔══╝██║╚══███╔╝██╔═══██╗██╔══██╗██║[/white]         
    [white]███║            ███║         ██║   ██║  ███╔╝ ██║   ██║██████╔╝██║[/white]         
    [white]█████████████████████████╗      ██║   ██║ ███╔╝  ██║   ██║██╔══██╗██║[/white]        
    [white]█████████████████████████║      ██║   ██║███████╗╚██████╔╝██║  ██║██║[/white]      
    [white]██████████╔═════█████████║      ╚═╝   ╚═╝╚══════╝ ╚═════╝ ╚═╝  ╚═╝╚═╝[/white]    
    [white]██████████║     █████████║[/white]       
    [white]████████████╗ ███████████║[/white]      [bright_white]Developer:[/bright_white] [green]Dhruv Shah[/green]     
    [white]████████████║ ███████████║[/white]      [bright_white]Github:[/bright_white] [blue]https://github.com/Dhruv9449[/blue]      
    [white]████████████║ ███████████║[/white]      [bright_white]Repository:[/bright_white] [blue]https://github.com/Dhruv9449/tizori-cli[/blue]         
    [white]█████████████████████████║[/white]         
    [white]█████████████████████████║[/white]             
    [white]╚════════════════════════╝[/white]       
    """
            )
            echo(ctx.get_help())

        # Check if the base URL is setit and command is not set-base-url
        if (
            config.get("base_url") == "" or config.get("base_url") is None
        ) and ctx.invoked_subcommand != "set-base-url":
            print("\n[red]Base URL not set![/red]")
            print("Please set the base URL using the command: [bold blue]tizori set-base-url <URL>[/bold blue]")
            raise Abort()

    except Exception as e:
        print(f"\n[red]Error:[/red] {str(e)}")


@app.command("set-base-url")
def set_base_url(url: str):
    """
    Set the base URL for the API
    """
    try:
        config = load_config()
        config["base_url"] = url
        save_config(config)
        print(f"\nBase URL set to: [bold blue]{url}[/bold blue]")
    except Exception as e:
        print(f"\n[red]Error:[/red] {str(e)}")


if __name__ == "__main__":
    try:
        app()
    except Exception as e:
        print(f"\n[red]An unexpected error occurred: {e}[/red]")
        raise Abort()



import requests
import typer
from rich import print


class BaseRequestWrapper:
    base_url = None
    bearer_token = None

    def __init__(self, base_url, bearer_token=None):    
        self.base_url = base_url + "/api/v1"
        self.bearer_token = bearer_token

    def set_bearer_token(self, bearer_token):
        self.bearer_token = bearer_token

    def get(self, endpoint, params=None):
        url = self.base_url + endpoint
        if self.bearer_token is not None:
            headers = {"Authorization": f"Bearer {self.bearer_token}"}
        else:
            headers = {}
        response = requests.get(url, headers=headers, params=params)
        return response.json(), response.status_code

    def post(self, endpoint, data=None):
        url = self.base_url + endpoint
        if self.bearer_token is not None:
            headers = {"Authorization": f"Bearer {self.bearer_token}"}
        else:
            headers = {}
        response = requests.post(url, headers=headers, json=data)
        return response.json(), response.status_code

    def patch(self, endpoint, data=None):
        url = self.base_url + endpoint
        if self.bearer_token is not None:
            headers = {"Authorization": f"Bearer {self.bearer_token}"}
        else:
            headers = {}
        response = requests.patch(url, headers=headers, json=data)
        return response.json(), response.status_code

    def delete(self, endpoint):
        url = self.base_url + endpoint
        if self.bearer_token is not None:
            headers = {"Authorization": f"Bearer {self.bearer_token}"}
        else:
            headers = {}
        response = requests.delete(url, headers=headers)
        return response.json(), response.status_code



from typer import Typer, Option
from rich import print
from typing_extensions import Annotated


from tizori_cli.wrapper import applications
from tizori_cli.commands.base import base_wrapper


applications_app = Typer()


@applications_app.command("list")
def list_applications():
    try:
        status, applications_list = applications.list_applications(base_wrapper)
        if status:
            print("\n[bold blue]Applications[/bold blue]")
            for application in applications_list:
                print(f"\nName: [green]{application.get('name')}[/green]")
                print(f"ID: [green]{application.get('id')}[/green]")
        else:
            print(f"\n[red]Error[/red]: {applications_list}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@applications_app.command("get-credentials")
def get_application_credentials(
    application_id: Annotated[str, Option(help="Application ID", prompt=True)],
):
    try:
        status, credentials = applications.get_application_credentials(base_wrapper, application_id)
        if status:
            print("\n[bold blue]Credentials[/bold blue]")
            print(f"Application: [green]{credentials.get('name')}[/green]")
            print(f"Username: [green]{credentials.get('credentials').get('username')}[/green]")
            print(f"Password: [green]{credentials.get('credentials').get('password')}[/green]")
        else:
            print(f"\n[red]Error[/red]: {credentials}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@applications_app.command("create")
def create_application(
    name: Annotated[str, Option(help="Name", prompt=True)],
):
    try:
        status, message = applications.create_application(base_wrapper, name)
        if status:
            print("\n[bold green]Application created successfully[/bold green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@applications_app.command("update-credentials")
def update_application_credentials(
    application_id: Annotated[str, Option(help="Application ID", prompt=True)],
    username: Annotated[str, Option(help="Client ID", prompt=True)],
    password: Annotated[str, Option(help="Client Secret", prompt=True)],
):
    try:
        data = {"username": username, "password": password}
        status, message = applications.update_application_credentials(base_wrapper, application_id, data)
        if status:
            print("\n[bold green]Credentials updated successfully![/bold green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@applications_app.command("delete")
def delete_application(
    application_id: Annotated[str, Option(help="Application ID", prompt=True)],
):
    try:
        status, message = applications.delete_application(base_wrapper, application_id)
        if status:
            print("\n[bold green]Application deleted successfully![/bold green]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")



from tizori_cli.config import set_config_value

def login(req_wrap, username, password):
    endpoint = "/auth/login"
    response, status = req_wrap.post(endpoint, data={"username": username, "password": password})
    if status == 200:
        token = response.get("token")
        set_config_value("token", token)
        return True, ""
    else:
        return False, response.get("detail", "Unknown Error")

def logout():
    set_config_value("token", "")
    return

def reset_password(req_wrap, username):
    endpoint = "/auth/reset-password"
    response, status = req_wrap.post(endpoint, data={"username": username})
    if status == 200:
        return True, response.get("password")
    else:
        return False, response.get("detail", "Unknown Error")


from typer import Typer, Option
from rich import print
from typing_extensions import Annotated


from tizori_cli.commands.base import base_wrapper
from tizori_cli.wrapper import auth

auth_app = Typer()


@auth_app.command("login")
def login(
    username: Annotated[str, Option(help="Tizori Username", prompt=True)],
    password: Annotated[str, Option(help="Tizori Password", prompt=True, hide_input=True)],
):
    try:
        status, message = auth.login(base_wrapper, username, password)
        if status:
            print("\n[bold green]Login Successful[/bold green]")
        else:
            print(f"\n[red]Login Failed[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@auth_app.command("logout")
def logout():
    try:
        auth.logout()
        print("\n[bold green]Logout Successful[/bold green]")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")


@auth_app.command("reset-password")
def reset_password(
    username: Annotated[str, Option(help="Tizori Username", prompt=True)],
):
    try:
        status, message = auth.reset_password(base_wrapper, username)
        if status:
            print("\n[green]Password Reset![/green]")
            # Display generated password
            print(f"Password: [bold blue]{message}[/bold blue]")
        else:
            print(f"\n[red]Error[/red]: {message}")
    except Exception as e:
        print(f"\n[red]Error[/red]: {str(e)}")
